-- Menu and parent "frame" for orbitals display.

-- Copyright 1995-2000 Tom Moran, may be used by anyone for any purpose.

with About_Dialog,
     Ada.Strings.Fixed,
     Claw,
     Claw.Edit,
     Claw.Frame_Window,
     Claw.Menus,
     Claw.Menus.Bar,
     Claw.Menus.Lists,
     Claw.Rectangles,
     Select_Dialog;
package body Control is

  type Menu_Options is (Quit,
                        Selection,
                        Do_3D,  -- or 2D
                        About,
                        Help);

  MENU_BASE : constant Claw.Menu_Identifier_Type :=
      Claw.Menus.Get_Unique_Menu_Identifier (
          Claw.Menu_Item_Index_Type(Menu_Options'Pos(Menu_Options'Last)+1));
          -- This call ensures that these Identifiers are unique in this run.

  Menu : Claw.Menus.Bar.Menu_Bar_Type;

  Control_Window : aliased Control_Window_Type;

  function Access_The_Control return Claw.Any_Window_Access_Type is
  begin
    if Is_Valid(Control_Window) then return Control_Window'access;
    else return null;
    end if;
  end Access_The_Control;

  Client_Area_Has_Changed : Boolean := False;

  function Client_Area_Changed return Boolean is
    Result : constant Boolean := Client_Area_Has_Changed;
  begin
    Client_Area_Has_Changed := False;
    return Result;
  end Client_Area_Changed;

  protected Client_Area_Info is
    function Get return Claw.Rectangle_Type;
    procedure Set(Area : in Claw.Rectangle_Type);
  private
    Control_Client_Area : Claw.Rectangle_Type
      := Claw.Frame_Window.Use_Default;
    -- Make an appropriate default even prior to Start being called.
  end Client_Area_Info;

  protected body Client_Area_Info is
    function Get return Claw.Rectangle_Type is
    begin
      Client_Area_Has_Changed := False;
      return Control_Client_Area;
    end Get;
    procedure Set(Area : in Claw.Rectangle_Type) is
    begin
      Control_Client_Area := Area;
      Client_Area_Has_Changed := True;
    end Set;
  end Client_Area_Info;

  function Client_Area return Claw.Rectangle_Type is
  begin
    return Client_Area_Info.Get;
  end Client_Area;

  function Head(Source : String; Count : Natural; Pad : Character:=' ')
  return String renames Ada.Strings.Fixed.Head;

  Help_Info : constant About_Dialog.Text_Array
    := ("    Shell 1s, 2p, 5d+1, 4f, 5g-3, etc.  ",
        "Initial green graph shows probability as",
        "a function of distance from center or   ",
        "latitude.  Bottom two tick marks are 1  ",
        "Bohr radius apart.  Move mouse while    ",
        "holding left button to rotate image.    ");

  About_Info : constant About_Dialog.Text_Array
    := ("Hydrogen electron probability cloud",
        "   (c) Tom Moran 1992-2000         ");


  procedure Start is
    use type Claw.Menu_Identifier_Type;
    use type Claw.Int;
  begin
    Menu := Claw.Menus.Lists.Create (
        (1=>Claw.Menus.Lists.Submenu ("&File",
           (1=>Claw.Menus.Lists.Menu_Item ("&Shell",   Menu_Options'Pos(Selection) + MENU_BASE),
            2=>Claw.Menus.Lists.Menu_Item ("E&xit",    Menu_Options'Pos(Quit) + MENU_BASE))),
         2=>Claw.Menus.Lists.Submenu ("&View",
           (1=>Claw.Menus.Lists.Menu_Item ("Blue-Red &3D", Menu_Options'Pos(Do_3D) + MENU_BASE,
                                        Kind => Claw.Menus.Checked))),
         3=>Claw.Menus.Lists.Submenu ("&Help",
           (1=>Claw.Menus.Lists.Menu_Item ("&Help", Menu_Options'Pos(Help) + MENU_BASE),
            2=>Claw.Menus.Lists.Menu_Item ("&About", Menu_Options'Pos(About) + MENU_BASE)))));
    Create (Window => Control_Window,
            Window_Name => "Atomic Orbitals",
            Menu => Menu,
            Exit_Id => Claw.Menu_Identifier_Type(
                       Menu_Options'Pos(Quit) + MENU_BASE));
    Show(Control_Window, Claw.Codes.Show_Startup);
    delay 0.3; -- give it time to paint, etc.
  end Start;

  subtype Shell_Name_Type is String(1 .. 4);

  Shell : Shell_Name_Type := "1s  ";

  function Get_Selection return String is
  begin
    return Ada.Strings.Fixed.Trim(Shell, Side => Ada.Strings.Both);
  end Get_Selection;

  procedure Put_Selection(S : in String) is
  begin
    Shell := S & (S'length+1 .. Shell'length => ' ');
    Wanted := Selection;
  end Put_Selection;

  subtype Dimensions is integer range 2 .. 3;
  Dimension : Dimensions := 2;

  function Want_Stereo return Boolean is
  begin
    return (Dimension = 3);
  end Want_Stereo;

  package Shell_Dialog renames Select_Dialog;

  procedure When_Command (Window : in out Control_Window_Type;
                          From   : in Claw.Command_Source_Type;
                          Command_Id : in Claw.Menu_Identifier_Type;
                          Unknown_Command : in out Boolean) is
    use type Claw.Menu_Identifier_Type;
  begin
    Unknown_Command := True; -- Until we know better.
    -- Process a menu selection.
    if Wanted = Unknown  -- don't accept new work till finished with old.
    and then Command_Id
       in MENU_BASE .. MENU_BASE + Menu_Options'Pos(Menu_Options'Last) then
        case Menu_Options'Val(Command_ID - MENU_BASE) is
            when Quit  =>
              Unknown_Command := False;  -- Should never get here.
            when Selection =>
              Unknown_Command := False;
              declare
                OK : Boolean;
              begin
                Shell_Dialog.Use_Dialog_Box(Window,
                  "Enter one of 1s, 2p, 5d+1, 4f, 5g-3, etc.",
                  OK, Shell);
                if OK then
                  Wanted := Selection;
                end if;
              end;
            when Do_3D =>
              Unknown_Command := False;
              if Dimension = 2 then
                Dimension := 3;
                Claw.Menus.Bar.Modify_Item_Name(Menu, Command_Id,
                  New_Name => "&2D");
              else
                Dimension := 2;
                Claw.Menus.Bar.Modify_Item_Name(Menu, Command_Id,
                  New_Name => "Blue-Red &3D");
              end if;
              Wanted := Change_Dimension;
            when About =>
              Unknown_Command := False;
              About_Dialog.Use_Dialog_Box(Window, "About Orbitals", About_Info);
            when Help =>
              Unknown_Command := False;
              About_Dialog.Use_Dialog_Box(Window, "Orbitals", Help_Info);
            when others =>
              null;
        end case;
    end if;
    if Unknown_Command then
        -- Call the original ("parent") routine, in case it handles
        -- the selection.
        Claw.Frame_Window.When_Command (Claw.Frame_Window.Frame_Window_Type(Window),
            From, Command_Id, Unknown_Command);
    end if;
  end When_Command;

  procedure When_Close(Window : in out Control_Window_Type) is
  begin
    Wanted := Close;
    Claw.Frame_Window.When_Close(Claw.Frame_Window.Frame_Window_Type(Window));
  end When_Close;

  procedure When_Resize (Window        : in out Control_Window_Type;
                         Resize_Params : in     Claw.Codes.Resize_Params_Type;
                         Do_Default    : in out Boolean) is
    use type Claw.Word;
    use type Claw.Int;
    Position : constant Claw.Point_Type
      := Claw.Rectangles.To_Position(Client_Area_Info.Get);
  begin
    Do_Default := True; -- Let any other processing happen.
    if Resize_Params.Client_Width > 0
    and then Resize_Params.Client_Height > 0 then
      Client_Area_Info.Set(Claw.Rectangles.To_Rectangle(
        Position => Position,
        Size => (Width  => Claw.Int(Resize_Params.Client_Width),
                 Height => Claw.Int(Resize_Params.Client_Height))));
      Client_Area_Has_Changed := True;
    end if;
  end When_Resize;

end Control;
