-- Copyright 1995-2000 Tom Moran, may be used by anyone for any purpose.

-- Display an animated, probabilistic electron "shell" of Hydrogen.
-- cf Mathematics and Wave Mechanics, Atkin, Wiley p. 235
--
--   We keep generating and displaying random electrons until
-- the mouse has moved, at which time the point cloud is frozen
-- and we simply wait for the display window to be closed, while
-- the Display package handles movement of the frozen cloud.
--
--   To flicker, we want to generate random electron positions,
-- the probability of a particular position being governed by
-- Schrodinger's wave equation for the Hydrogen atom.  Luckily,
-- for this case, the probabilities as a function of radial
-- distance from the nucleus (R) and as a function of latitude
-- (Phi), are independent, and all longitudes (Theta) are
-- equally likely.  So we need merely generate an R from the
-- distribution of radial probabilities, a Phi from the
-- distribution for Phi, and any arbitrary Theta.
--
--   To create a "sampling bucket" of R values with the correct
-- distribution, we find the cumulative probability less than r
-- for various r's, and then we populate an "inverse" array with
-- the appropriate number of occurrences of those r's.
-- Similarly for Phi.

with Ada.Command_Line,
     Ada.Exceptions,
     Ada.Numerics,
     Ada.Numerics.Discrete_Random,
     Ada.Numerics.Elementary_Functions,
     Ada.Numerics.Float_Random,
     Ada.Text_IO,
     Claw,
     Claw.Basic_Window,
     Claw.Message_Box,
     Claw.Rectangles,
     Claw.Window_Operations,
     Control,
     Display,
     Hydrogen_Atom;

procedure Orbitals is
  use Ada.Numerics;
  use Ada.Numerics.Elementary_Functions;

  procedure Get_Parameters(Name : in     String;
                           OK   :    out Boolean;
                           N, L, S : out Integer) is
  -- if OK, then N, L, and S have legal values
    S_Sign:integer;
    S_Char:character;
  begin
    OK := False;
    if Name'length < 2
    or else Name(Name'first) not in '1' .. '9' then
      return;
    end if;
    case Name(Name'first+1) is
      when 'S' | 's' => L := 0;
      when 'P' | 'p' => L := 1;
      when 'D' | 'd' => L := 2;
      when 'F' | 'f' => L := 3;
      when 'G' | 'g' => L := 4;
      when others => return;
    end case;
    N:=character'pos(Name(Name'first))-character'pos('0');
    if N <= L then return;end if;
    if Name'length > 2 then
      if Name'length = 4 then
        if Name(Name'first+2) = '-' then S_Sign:= -1;
        elsif Name(Name'first+2) = '+' then S_Sign:= +1;
        else
          return;
        end if;
        S_Char:=Name(Name'first+3);
      else
        return;
      end if;
      if S_Char not in '0' .. '9' then return;end if;
      S:=character'pos(S_Char)-character'pos('0');
      if S > L then return;end if;
      S:=S_Sign*S;
    else
      S:=0;
    end if;
    OK := True;
  end Get_Parameters;

  function Shell_Name(N, L, S : integer) return String is
    L_Name : constant String := "spdfg";
    Digit : constant String := "123456789";
  begin
    if S = 0 then
      return Digit(N) & L_Name(L+1);
    elsif S > 0 then
      return Digit(N) & L_Name(L+1) & '+' & Digit(S);
    else
      return Digit(N) & L_Name(L+1) & '-' & Digit(-S);
    end if;
  end Shell_Name;

  function Cos(Theta:in float) return float
  renames Ada.Numerics.Elementary_Functions.Cos;

  function Sin(Theta:in float) return float
  renames Ada.Numerics.Elementary_Functions.Sin;

  subtype Interval_Counts is integer range 0 .. 400;
  type Cum_Probs is array(Interval_Counts range <>) of float;

  subtype Inverted_Cum_Indices is integer range 0 .. 16383;
  -- The more entries, the greater the resolution.
  type Inverted_Cum_Probs is array(Inverted_Cum_Indices) of Interval_Counts;

  procedure Invert_Cum_Prob_Histogram(Cum : in     Cum_Probs;
                                      Inv :    out Inverted_Cum_Probs) is
    Cum_I:Interval_Counts :=Cum'first;
  begin
    for i in Inv'range loop
      for ci in Cum_I .. Cum'last loop
        if float(i)/float(Inv'last) <= Cum(ci) then
          Inv(i) := ci;
          Cum_I := ci;
          exit;
        end if;
      end loop;
    end loop;
  end Invert_Cum_Prob_Histogram;

  Radial_Cum_Prob:Cum_Probs(0 .. 200);
  Phi_Cum_Prob :Cum_Probs(0 .. 360);

  Inv_Radial_Cum_Prob:Inverted_Cum_Probs;
  Inv_Phi_Cum_Prob:Inverted_Cum_Probs;

  Random_Generator : Ada.Numerics.Float_Random.Generator;

  package Random_ICP is new Ada.Numerics.Discrete_Random(Inverted_Cum_Indices);
  Random_ICP_Gen : Random_ICP.Generator;

  procedure Generate_Position(X,Y,Z:out Display.Unit_Cube_locations) is
  -- Generate random R, Phi, Theta around the center of the unit cube,
  -- using appropriate probability distributions.
  -- Return the point's Cartesian coordinates.
    use Ada.Numerics;
    R:float:=0.0;
    Phi:float:=0.0;
    Theta:float:=0.0;
    Sin_Phi:float;
  begin
    R:= float(Inv_Radial_Cum_Prob(Random_ICP.Random(Random_ICP_Gen)))
        /float(2*Radial_Cum_Prob'last);
    Phi:=pi*float(Inv_Phi_Cum_Prob(Random_ICP.Random(Random_ICP_Gen)))
            /float(Phi_Cum_Prob'last);
    Theta:=2.0*pi*Float_Random.Random(Random_Generator);
    Sin_Phi:=Sin(Phi);
    X:=R*Sin_Phi*Sin(Theta)+0.5;
    Y:=R*Cos(Phi)+0.5;
    Z:=R*Sin_Phi*Cos(Theta)+0.5;
  end Generate_Position;

  procedure Display_A_Shell(Stereo  : in Boolean;
                            N, L, S : in Integer) is
    use type Control.Wanted_Type;
    use type Claw.Rectangle_Type;
    use type Claw.Any_Window_Access_Type;
    use type Claw.Int;
    package Atom is new Hydrogen_Atom(N,L,S);

    Cloud : Display.Cloud_Display_Type(Max_Particle_Count=>300);

    Annotation : Display.XY_Pair_Lists(1 .. 2*Radial_Cum_Prob'length
                                            +2*Phi_Cum_Prob'length
                                            +16); -- two ticks @ 8 dots
    Ann_Count : integer := 0;

    Max_Radial:float;
    Unit_Cube_Width:float;   -- := float(2*Bohr_radii_to_show)*a0;

    procedure Find_Radial_Scale is
      Half_Bohr_Radii_To_Check:positive:=20*(L+1)*integer(N);
      X:array(0 .. 2*Half_Bohr_Radii_To_Check) of float;
      Farthest_Diameter:natural:= 1;  -- with significant probability
    begin
      Unit_Cube_Width:=Atom.a0*float(X'last);
      Max_Radial := 0.0;
      for ir in X'range loop
        X(ir):=Atom.Radial(Unit_Cube_Width*(float(ir)/float(2*X'last)))**2;
        if X(ir) > Max_Radial then Max_Radial:=X(ir);end if;
      end loop;
      for ir in X'range loop
        if X(ir)/Max_Radial > 0.002 then Farthest_Diameter:=ir;end if;
      end loop;
      Unit_Cube_Width:=float(integer'max(1,Farthest_Diameter))*Atom.a0;
    end Find_Radial_Scale;

    procedure Calculate_Radial is
      Cum_Prob:float:=0.0;
      R_Part:float;
      R:float;
    begin
      for ir in Radial_Cum_Prob'range loop
        R:=float(ir)/float(2*Radial_Cum_Prob'last);
        R_Part:=Atom.Radial(R*Unit_Cube_Width)**2/Max_Radial;
        Cum_Prob:=Cum_Prob+R_Part;
        Radial_Cum_Prob(ir):=Cum_Prob;
        if R_Part in Display.Unit_Cube_locations then
          Ann_Count := Ann_Count+1;
          Annotation(Ann_Count) :=(X=>0.5+R, Y=>1.0-R_Part);
          Ann_Count := Ann_Count+1;
          Annotation(Ann_Count) :=(X=>0.5-R, Y=>1.0-R_Part);
        end if;
      end loop;
      for ir in Radial_Cum_Prob'range loop
        Radial_Cum_Prob(ir):=Radial_Cum_Prob(ir)/Cum_Prob;
      end loop;
      Invert_Cum_Prob_Histogram(Radial_Cum_Prob, Inv_Radial_Cum_Prob);
    end Calculate_Radial;

    procedure Calculate_Angular is
      T_Part:float;
      Phi:float;
      Max_T_Part:float:=0.0;
      Cum_Prob:float:=0.0;
      X,Y:float;
      V:array(Phi_Cum_Prob'range) of float;
    begin
      for itheta in Phi_Cum_Prob'range loop
        Phi:=float(itheta)/float(Phi_Cum_Prob'last)*Ada.Numerics.pi;
        T_Part:=Atom.Angular(Phi)**2;
        Cum_Prob:=Cum_Prob+T_Part;
        Phi_Cum_Prob(itheta):=Cum_Prob;
        if T_Part > Max_T_Part then Max_T_Part:=T_Part;end if;
        V(itheta):=T_Part;
      end loop;
      for itheta in Phi_Cum_Prob'range loop
        Phi:=float(itheta)/float(Phi_Cum_Prob'last)*Ada.Numerics.pi;
        Phi_Cum_Prob(itheta):=Phi_Cum_Prob(itheta)/Cum_Prob;
        T_Part:=V(itheta)/Max_T_Part;
        Y:=T_Part*Cos(Phi)/2.0+0.5;
        X:=T_Part*Sin(Phi)/2.0+0.5;
        if X in Display.Unit_Cube_Locations
        and Y in Display.Unit_Cube_Locations then
          Ann_Count := Ann_Count+1;
          Annotation(Ann_Count) :=(x=>X, y=>1.0-Y);
          Ann_Count := Ann_Count+1;
          Annotation(Ann_Count) :=(x=>1.0-X, y=>1.0-Y);
        end if;
      end loop;
      Invert_Cum_Prob_Histogram(Phi_Cum_Prob, Inv_Phi_Cum_Prob);
    end Calculate_Angular;

    procedure Calculate_Length_Ticks is
      Tick_Width : constant float := Atom.a0/Unit_Cube_Width;
    begin
      if Tick_Width < 0.5 then
        for i in 1 .. 8 loop
          Ann_Count:=Ann_Count+1;
          Annotation(Ann_Count) := (x=>0.5, y=> 1.0-float(i)/400.0);
          Ann_Count:=Ann_Count+1;
          Annotation(Ann_Count) := (x=>0.5+Tick_Width, y=> 1.0-float(i)/400.0);
        end loop;
      end if;
    end Calculate_Length_Ticks;

    Buffer : Display.Point_Lists(0 .. 7);
    Buffer_Index : Integer := 0;

  begin
    Find_Radial_Scale;
    Calculate_Radial;
    Calculate_Angular;
    Calculate_Length_Ticks;
    Display.Open(Cloud, "Atomic Orbitals", Annotation(1 .. Ann_Count),
                 Stereo, Parent => Control.Access_The_Control);
    Claw.Basic_Window.Set_Name(Claw.Basic_Window.Basic_Window_Type(
                                 Control.Access_The_Control.all),
                               Shell_Name(N, L, S) & " orbital");

    loop
      if not Display.Mouse_Has_Moved(Cloud) then
        declare
          X,Y,Z:Display.Unit_Cube_Locations;
        begin
          Generate_Position(X,Y,Z);
          Buffer(Buffer_Index) := (X,Y,Z);
          Buffer_Index := (Buffer_Index+1) mod Buffer'length;
          if Buffer_Index = 0 then
            Display.Brighten(Cloud, Buffer);
          end if;
        end;
      end if;
      Claw.Yield;
      if Display.Is_Closed(Cloud) then return;end if;
      if Control.Client_Area_Changed then
        declare
          Display_Rectangle : constant Claw.Rectangle_Type := Control.Client_Area;
        begin
          Display.Move(Cloud,
            Position => (0,0), -- top left in client area
            Size => (Width => Display_Rectangle.Right - Display_Rectangle.Left,
                     Height=> Display_Rectangle.Bottom - Display_Rectangle.Top),
            Redraw   => True);
            Display.Full_Redraw(Cloud);
        exception
          when Claw.Not_Valid_Error => return; -- Perhaps Display is now closed?
          when Claw.Windows_Error => null;
        end;
      end if;
      exit when Control.Wanted /= Control.Unknown;
    end loop;
    Display.Close(Cloud);
  exception
    when oops:others =>
      Display.Close(Cloud);
      Claw.Message_Box.Message_Box(Text=>
           Ada.Exceptions.Exception_Information(oops),
         Caption=>"Error",
         Flags=>Claw.Message_Box.Make_Flags);
  end Display_A_Shell;

  procedure Run_Per_Control is
    OK : Boolean;
    N : Integer := 1;    -- default is 1s
    L, S : Integer := 0;
    -- note a shell to display if user makes illegal selection
    Previous_N : Integer;
    Previous_L : Integer;
    Previous_S : Integer;
  begin
    loop
      case Control.Wanted is
        when Control.Unknown =>
          Claw.Yield;
        when Control.Selection =>
          Previous_N := N;
          Previous_L := L;
          Previous_S := S;
          Get_Parameters(Control.Get_Selection, OK, N, L, S);
          if not OK then
            Claw.Message_Box.Message_Box(Text=>"Impossible Shell",
                                         Caption=>"Error",
                                         Flags=>Claw.Message_Box.Make_Flags);
            N := Previous_N;  -- restore previous, legit, shell
            L := Previous_L;
            S := Previous_S;
          end if;
          Control.Wanted := Control.Unknown;
          Display_A_Shell(Control.Want_Stereo, N, L, S);
        when Control.Change_Dimension =>
          Control.Wanted := Control.Unknown;
          Display_A_Shell(Control.Want_Stereo, N, L, S);
        when Control.Close =>
          exit;
      end case;
    end loop;
  end Run_Per_Control;

begin
  if Ada.Command_Line.Argument_Count = 1 then
    Control.Put_Selection(Ada.Command_Line.Argument(1));
  else
    Control.Put_Selection("1s");
  end if;
  Control.Start;
  Run_Per_Control;

end Orbitals;
